#!/bin/bash -u

# setup args in the right order for making getopt evaluation
# nice and easy.  You'll need to read the manpages for more info
args=$(getopt -o h -- "$@")
eval set -- "$args"

###########################################################
usage() {
    echo ''           >&2
    echo "$0 <%post number>" >&2
    echo "   This script is really just an easy to call" >&2
    echo "   wrapper around authselect/authconfig" >&2
    echo ''           >&2
    echo "   $0 0 # disable our config" >&2
    echo "   $0 1 # (or larger) enable our config" >&2
    exit 1
}

###########################################################
for arg in $@; do
    case $1 in
        -- )
            # end of getopt args, shift off the -- and get out of the loop
            shift
            break 2
           ;;
         -h )
            # get help
            usage
           ;;

    esac
done

###########################################################

CURRENT_INSTALLS_OF_THIS_PACKAGE=${1:-0}

NOW=$(date +%x_%T)
TMPFILE=$(mktemp)

if [[ ! -e /etc/os-release ]]; then
    echo "Could not find /etc/os-release" >&2
    exit 1
fi

if [[ ! -d /etc/sssd ]]; then
    echo "Could not find '/etc/sssd/' directory" >&2
    echo "  is sssd installed? (yum install sssd)" >&2
    exit 1
fi

which augtool >/dev/null 2>&1
if [[ $? -ne 0 ]]; then
    echo "Could not find augtool" >&2
    echo "PATH contains: $PATH" >&2
    exit 1
fi

source /etc/os-release
MAJOR_VERSION=$(echo ${VERSION_ID} | cut -d'.' -f1)

if [[ ${CURRENT_INSTALLS_OF_THIS_PACKAGE} -eq 0 ]]; then
    echo "only remove the config on removing this rpm, not upgrading" >/dev/null
    # only remove the config on removing this rpm, not upgrading
    logger -p auth.info 'Removing fermilab-conf_sssd'
    touch /etc/sssd/sssd.conf
    chmod 600 /etc/sssd/sssd.conf
    restorecon -Rf /etc/sssd/ >/dev/null 2>&1
    grep -q '^\[sssd\]' /etc/sssd/sssd.conf
    if [[ $? -ne 0 ]]; then
        echo "[sssd]" >> /etc/sssd/sssd.conf
    fi

    grep -q '^\[pam\]' /etc/sssd/sssd.conf
    if [[ $? -ne 0 ]]; then
        echo "[pam]" >> /etc/sssd/sssd.conf
    fi

    grep -q '^\[nss\]' /etc/sssd/sssd.conf
    if [[ $? -ne 0 ]]; then
        echo "[nss]" >> /etc/sssd/sssd.conf
    fi

    cat > ${TMPFILE} <<EOF
match /files/etc/sssd/sssd.conf/*[. = "sssd"]/domains
EOF
    MATCHES=$(augtool <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi

    echo ${MATCHES} | grep -q 'FNAL'
    if [[ $? -eq 0 ]]; then
        echo "remove domain from list" >/dev/null
        # remove domain from list
        DOMAINS=$(echo ${MATCHES} | cut -d= -f2)
        if [[ "${DOMAINS}" != 'FNAL' ]]; then
            DOMAINS=$(echo ${DOMAINS} | sed -e 's/FNAL//')
            DOMAINS=$(echo ${DOMAINS} | sed -e 's/,,//')
            DOMAINS=$(echo ${DOMAINS} | sed -e 's/ *$//')
            DOMAINS=$(echo ${DOMAINS} | sed -e 's/,*$//')
            cat > ${TMPFILE} <<EOF
set /files/etc/sssd/sssd.conf/*[. = 'sssd']/domains ${DOMAINS}
EOF
            OUT=$(augtool -s <${TMPFILE})
            if [[ $? -ne 0 ]]; then
                exit 1
            fi
            systemctl restart sssd > /dev/null 2>&1
        else
            cat > ${TMPFILE} <<EOF
rm /files/etc/sssd/sssd.conf/*[. = "sssd"]/domains
EOF
            OUT=$(augtool -s <${TMPFILE})
            systemctl disable sssd > /dev/null 2>&1
            systemctl stop sssd > /dev/null 2>&1
        fi
    fi

else

    # Not sure why these cleanups are needed.... but eh
    (
     mkdir -p /var/log/sssd
     touch /var/log/sssd/sssd_nss.log /var/log/sssd/sssd_pam.log /var/log/sssd/sssd.log
     chmod 600 /var/log/sssd/sssd_nss.log /var/log/sssd/sssd_pam.log /var/log/sssd/sssd.log
     chown sssd:sssd /var/log/sssd /var/log/sssd/sssd_nss.log /var/log/sssd/sssd_pam.log
     restorecon -Rf /var/log/sssd/
    ) &

    # setup the basics, skip name resolution in case it is broken
    if [[ -e /etc/sssd/sssd.conf ]]; then
       cp /etc/sssd/sssd.conf /run/user/0 || true
    fi
    touch /etc/sssd/sssd.conf
    chmod 600 /etc/sssd/sssd.conf
    chown 0:0 /etc/sssd/sssd.conf

    logger -p auth.info 'Installing/Upgrading fermilab-conf_sssd'

    grep -q '^\[sssd\]' /etc/sssd/sssd.conf
    if [[ $? -ne 0 ]]; then
        echo "[sssd]" >> /etc/sssd/sssd.conf
    fi

    grep -q '^\[pam\]' /etc/sssd/sssd.conf
    if [[ $? -ne 0 ]]; then
        echo "[pam]" >> /etc/sssd/sssd.conf
    fi

    grep -q '^\[nss\]' /etc/sssd/sssd.conf
    if [[ $? -ne 0 ]]; then
        echo "[nss]" >> /etc/sssd/sssd.conf
    fi

    # permit "extra" nss files (passwd_file/group_file)
    echo 'permit "extra" nss files (passwd_file/group_file)' >/dev/null
    cat > ${TMPFILE} <<EOF
set /files/etc/sssd/sssd.conf/*[. = "nss"]/pwfield *
EOF
    OUT=$(augtool -s <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi

    # enable the domain
    echo "enable the domain" >/dev/null
    cat > ${TMPFILE} <<EOF
match /files/etc/sssd/sssd.conf/*[. = "sssd"]/domains
EOF
    MATCHES=$(augtool <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi

    echo "insert the domain" >/dev/null
    echo ${MATCHES} | grep -qe '(no matches)\|(none)'
    if [[ $? -eq 0 ]]; then
        cat > ${TMPFILE} <<EOF
set /files/etc/sssd/sssd.conf/*[. = "sssd"]/domains FNAL
EOF
        OUT=$(augtool -s <${TMPFILE})
        if [[ $? -ne 0 ]]; then
            exit 1
        fi
    else
        echo ${MATCHES} | grep -q 'FNAL'
        if [[ $? -ne 0 ]]; then
            DOMAINS=$(echo ${MATCHES} | cut -d= -f2)
            cat > ${TMPFILE} <<EOF
set /files/etc/sssd/sssd.conf/*[. = 'sssd']/domains ${DOMAINS}, FNAL
EOF
            OUT=$(augtool -s <${TMPFILE})
            if [[ $? -ne 0 ]]; then
                exit 1
            fi

        fi
    fi

    # enable domain services (if needed)
    echo "enable domain services (if needed)" >/dev/null
    cat > ${TMPFILE} <<EOF
match /files/etc/sssd/sssd.conf/*[. = 'sssd']/services
EOF
    MATCHES=$(augtool <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi

    echo ${MATCHES} | grep -qe '(no matches)\|(none)'
    if [[ $? -eq 0 ]]; then
        if [[ ${MAJOR_VERSION} -lt 8 ]]; then
            cat > ${TMPFILE} <<EOF
set /files/etc/sssd/sssd.conf/*[. = "sssd"]/services "nss, pam"
EOF
            OUT=$(augtool -s <${TMPFILE})
            if [[ $? -ne 0 ]]; then
                exit 1
            fi
        else
            # EL8+ uses socket activated, no need to list here
            true
        fi
    else
        if [[ ${MAJOR_VERSION} -lt 8 ]]; then
            SERVICES='nss,pam'
            SERVICE_LIST=$(echo ${MATCHES} | cut -d= -f2)
            SERVICE_LIST="${SERVICES}, pam, nss"
            SERVICE_LIST=$(echo ${SERVICES} | tr ' ' '\012' | tr -d ',' | sort -u)
            for element in ${SERVICE_LIST}; do
                echo $element | grep -q autofs
                if [[ $? -eq 0 ]]; then
                    SERVICES="${SERVICES},autofs"
                    continue
                fi
                echo $element | grep -q ifp
                if [[ $? -eq 0 ]]; then
                    SERVICES="${SERVICES},ifp"
                    continue
                fi
                echo $element | grep -q sudo
                if [[ $? -eq 0 ]]; then
                    SERVICES="${SERVICES},sudo"
                    continue
                fi
                echo $element | grep -q ssh
                if [[ $? -eq 0 ]]; then
                    SERVICES="${SERVICES},ssh"
                    continue
                fi
                echo $element | grep -q pac
                if [[ $? -eq 0 ]]; then
                    SERVICES="${SERVICES},pac"
                    continue
                fi
            done
            cat > ${TMPFILE} <<EOF
set /files/etc/sssd/sssd.conf/*[. = "sssd"]/services "${SERVICES}"
EOF
            OUT=$(augtool -s <${TMPFILE})
            if [[ $? -ne 0 ]]; then
                exit 1
            fi
        else
            # EL8+ uses socket activated, no need to list here
            cat > ${TMPFILE} <<EOF
rm /files/etc/sssd/sssd.conf/*[. = "sssd"]/services
EOF
            OUT=$(augtool -s <${TMPFILE})
            if [[ $? -ne 0 ]]; then
                exit 1
            fi
        fi
    fi

    which authselect >/dev/null 2>&1
    if [[ $? -eq 0 ]]; then
        echo "Found authselect" >/dev/null
        OUT=$(authselect select sssd 2>&1)
        if [[ $? -eq 4 ]]; then
            RUN_AUTHSELECT_FORCE=0
            # look at the config files in 'unknown state'
            for filename in $(for i in $(echo ${OUT} | tr -d '[' | tr -d ']'); do echo ${i} | grep etc; done); do
                # This notice should be sufficient to justify running authselect
                grep -q 'User changes will be destroyed the next time authselect is run' ${filename}
                if [[ $? -ne 0 ]]; then
                    if [[ "${filename}" == '/etc/nsswitch.conf' ]]; then
                        # I can't think of a good way to directly check nsswitch.conf to make sure it
                        #  is safe to replace it probably is....
                        true
                    else
                        RUN_AUTHSELECT_FORCE=1
                        echo "File [${filename}] has local changes, you must manually configure your system to use sssd" >&2
                        logger -p auth.warning "Not running authselect because ${filename} has local changes"
                    fi
                fi
            done

            if [[ ${RUN_AUTHSELECT_FORCE} -eq 0 ]]; then
                OUT=$(authselect select sssd --force)
                logger -p auth.warning "overwriting local config with 'authselect select sssd --force'"
            fi
        else
            logger -p auth.info 'ran authselect select sssd'
        fi
    else
        which authconfig >/dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            echo "Found authconfig" >/dev/null
            OUT=$(authconfig --update --enablesssdauth --savebackup=${NOW}_fermilab-conf_sssd)
            logger -p auth.info 'ran authconfig --update --enablesssdauth'
        fi
    fi

    restorecon -Rf /etc/sssd/ >/dev/null 2>&1

    systemctl daemon-reload >/dev/null 2>&1

    logger -p auth.debug 'restarting sssd to apply changes'
    if [[ ${MAJOR_VERSION} -lt 8 ]]; then
        systemctl disable sssd-nss.socket sssd-pam.socket >/dev/null 2>&1
        systemctl enable sssd-nss.service sssd-pam.service >/dev/null 2>&1
        systemctl stop sssd-nss.socket sssd-pam.socket >/dev/null 2>&1
        systemctl restart sssd-nss.service sssd-pam.service >/dev/null 2>&1
    else
        systemctl disable sssd-nss.service sssd-pam.service >/dev/null 2>&1
        systemctl enable sssd-nss.socket sssd-pam.socket >/dev/null 2>&1
        systemctl start sssd-nss.socket sssd-pam.socket >/dev/null 2>&1 && \
        systemctl restart sssd.service >/dev/null 2>&1
    fi

    systemctl restart sssd.service >/dev/null 2>&1
    systemctl enable sssd.service >/dev/null 2>&1

fi

rm -f ${TMPFILE}
